package net.ericaro.neobin;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/** Business Object containing all the information described in a neobin file.
 * 
 * A Neobin is a graph of State and Transitions.
 * 
 * States are node in the regular expression graph. They represent a choice in the file format.
 * Transitions are vertices, and represent a set of fixed size data.
 * 
 * 
 * @author eric
 *
 */
public class BinaryFormat {

	String packageName;  // package name ( used both as the target for generation, and as a unique namespace for languages
	private String	expression; // the source regexp (for debug)
	List<State> states; // list of built states from the regexp
	List<Transition> transitions; // list of built transitions from the regexp
	private State	start; // the starting state

	// static list of used/available primitive types. But why?
	EnumSet<PrimitiveType> types = EnumSet.range(PrimitiveType.BYTE, PrimitiveType.DOUBLE);
	
	/** full constructor, kept protected because it is better to use the binaryformat builder.
	 * 
	 * @param packageName
	 * @param start
	 * @param states
	 * @param transitions
	 */
	BinaryFormat(String packageName, String expression, State start, Collection<State> states, Collection<Transition> transitions) {
		super();
		this.packageName = packageName;
		this.expression = expression ;
		this.start = start;
		this.states = new ArrayList<State>(states);
		this.transitions = new ArrayList<Transition>(transitions);
	}

	/** returns the list of states
	 * 
	 * @return
	 */
	public Iterable<State> getStates() {
		return states;
	}

	/** return all transitions
	 * 
	 * @return
	 */
	public Iterable<Transition> getTransitions() {
		return transitions;
	}

	
	public String getPackageName() {
		return packageName;
	}


	/** return the starting state.
	 * 
	 * @return
	 */
	public State getStart() {
		return start;
	}
	
	/** transition can share the same name. For instance "float" can happen both
	 * after an "int", and another "float" in this example : "int, float+ "
	 * Hence, this methods return the iterable over unique transitions, i.e. with unique name.
	 * 
	 * @return
	 */
	public Iterable<Transition> getUniqueTransitions(){
		List<Transition> res = new ArrayList<Transition>();
		Set<String> names = new HashSet<String>();
		for(Transition t: transitions)
			if (names.add(t.getMethod()))
				res.add(t);
		return res;
	}
	
	/** return available primitive types.
	 * 
	 * @return
	 */
	public Set<PrimitiveType> getTypes() {
		return types;
	}

	public String toSmallDescription() {
		StringBuilder sb = new StringBuilder();
		sb.append("neobin/").append(packageName).append("->").append(expression);
		return sb.toString();
	}
	
	
}
